{---------------------------------------------------------------------
  Print It

  Twocolumn printing of texts, especially Delphi-sources, with a
  fixed page setup. Each page has 104 lines, each line is 72 chars
  long. Too long lines are cut off, this can be seen by '' at the
  line's end. The font courier is used.

  Numbers are adapted to A4 paper format.

  Usage A: Start program and select the file to print.

  Usage B: Send one ore more files per drag&drop to the program.

  rk 09/01/98
 ---------------------------------------------------------------------}

Unit PrintIt1;

Interface

{---------------------------------------------------------------------}

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Printers, StdCtrls, ExtCtrls;

{---------------------------------------------------------------------}

Type
  TForm1 = Class(TForm)
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    Procedure Button1Click(Sender: TObject);
    Procedure FormCreate(Sender: TObject);
    Procedure FormClose(Sender: TObject; Var Action: TCloseAction);
  Private
    StringList : TstringList;

    CharsPerLine, LinesPerPage, NumberOfPages,
    { the units of the following variables are printer-pixels }
    BorderLeft, BorderRight, BorderTop, BorderBottom,
    HeaderFooterHeight, xLine,
    xPos, xPosLeft, xPosRight,
    yPos, yPosTop, yDelta : Integer;

    TheTime, FileName : String;

    Procedure SetLineLength;
    Procedure PerformSettings;
    Procedure Print;
    Procedure Header;
    Procedure Footer;
    Procedure Line;
    Procedure PrintJob( TheFileName : String );
  End;

{---------------------------------------------------------------------}
Var Form1 : TForm1;
{---------------------------------------------------------------------}

Implementation

{$R *.DFM}

{---------------------------------------------------------------------}
Procedure TForm1.Button1Click(Sender: TObject);
{---------------------------------------------------------------------}
Begin
  If OpenDialog1.Execute Then PrintJob( OpenDialog1.FileName );
End;

{---------------------------------------------------------------------}
Procedure TForm1.SetLineLength;
{---------------------------------------------------------------------}
Var i : Integer;
    s : String;
Begin
  For i := 0 To StringList.Count-1 Do Begin
    If Length( StringList.Strings[i] ) > CharsPerLine Then Begin
      s := Copy( StringList.Strings[i], 1, CharsPerLine-1 );
      StringList.Strings[i] := s + '';
    End;
  End;
End;

{---------------------------------------------------------------------}
Procedure TForm1.PerformSettings;
{---------------------------------------------------------------------}
Begin
  NumberOfPages := Trunc( StringList.Count / (2*LinesPerPage) ) + 1;
End;

{---------------------------------------------------------------------}
Procedure TForm1.Print;
{---------------------------------------------------------------------}
Var Counter, i : Integer;
Begin
  Printer.BeginDoc;
  Printer.Canvas.Font.Name := 'Courier';
  Printer.Canvas.Font.Size := 6;

  TheTime := FormatDateTime( 'dd.mm.yyyy  tt', now );
  Counter := 0;

  For i:=1 To StringList.Count Do Begin

    If Counter = 0 Then Begin
      Printer.Canvas.Font.Size := 18;
      Printer.Canvas.Font.Style := [fsBold];
      Header; Footer; Line;
      Printer.Canvas.Font.Size := 6;
      Printer.Canvas.Font.Style := [];
      xPos := xPosLeft;
      yPos := yPosTop;
    End;

    Printer.Canvas.TextOut( xPos, yPos, StringList.Strings[i-1] );

    Inc ( Counter );
    yPos := yPos + yDelta;

    If Counter = LinesPerPage Then Begin
      xPos := xPosRight;
      yPos := YPosTop;
    End;

    If Counter = 2*LinesPerPage Then Begin
      Printer.NewPage;
      Counter := 0;
    End;

  End;

  Printer.EndDoc;
End;

{---------------------------------------------------------------------}
Procedure TForm1.Header;
{---------------------------------------------------------------------}
Var x, y : Integer;
Begin
  x := xLine  - Printer.Canvas.TextWidth( FileName ) Div 2;
  y := BorderTop + HeaderFooterHeight Div 2;
  Printer.Canvas.TextOut( x, y, FileName );
End;

{---------------------------------------------------------------------}
Procedure TForm1.Footer;
{---------------------------------------------------------------------}
Var x, y : Integer;
    s    : String;
Begin
  x := 0;
  y := Printer.PageHeight - BorderTop - HeaderFooterHeight Div 2;
  Printer.Canvas.TextOut( x, y, TheTime );

  s := Format( 'Page: %d of %d', [Printer.PageNumber,NumberOfPages] );
  x := Printer.PageWidth - BorderRight - Printer.Canvas.TextWidth( s );
  Printer.Canvas.TextOut( x, y, s );
End;

{---------------------------------------------------------------------}
Procedure TForm1.Line;
{---------------------------------------------------------------------}
Var y1, y2 : Integer;
Begin
  y1 := BorderTop + HeaderFooterHeight;
  y2 := Printer.PageHeight - BorderBottom - HeaderFooterHeight;
  Printer.Canvas.MoveTo( xLine, y1 );
  Printer.Canvas.LineTo( xLine, y2 );
End;

{---------------------------------------------------------------------}
Procedure TForm1.FormCreate( Sender: TObject );
{---------------------------------------------------------------------}
Const xDelta = 50;
Var SearchRec : TSearchRec;
    i         : Integer;
    Path      : String;
Begin
  StringList := TStringList.Create;

  CharsPerLine        := 72;
  LinesPerPage        := 104;

  BorderLeft          := Trunc( 0.2 * 600 );
  BorderRight         := Trunc( 0.2 * 600 );
  BorderTop           := 0;
  BorderBottom        := 0;
  HeaderFooterHeight  := Trunc( 0.6 * 600 );

  Printer.Orientation := poPortrait;

  xPosLeft  := BorderLeft;
  xLine     := ( Printer.PageWidth + BorderLeft - BorderRight ) Div 2;
  xPosRight := xLine + xDelta;
  yDelta    := Trunc(
    ( Printer.PageHeight - BorderTop - BorderBottom
      - 2* HeaderFooterHeight )
    / LinesPerPage );
  yPosTop   := BorderTop + HeaderFooterHeight + yDelta;

  {-------------------------------------------------------------------
    Do all jobs, if more files are in the line. Drag & drop !
   -------------------------------------------------------------------}
  If ParamCount > 0 Then Begin
    For i := 1 To ParamCount Do Begin
      Path := ExtractFilePath( ParamStr(i) );
      If FindFirst( ParamStr(i), faAnyFile, SearchRec ) = 0 Then
        PrintJob( Path + SearchRec.Name );
      FindClose( SearchRec );
    End;
    Application.Terminate;
  End;
End;

{---------------------------------------------------------------------}
Procedure TForm1.PrintJob( TheFileName : String );
{---------------------------------------------------------------------}
Begin
{  MessageDlg( TheFileName, mtInformation, [mbOk], 0);}
  If FileExists( TheFileName ) Then Begin
    Try
{      MessageDlg( 'exists', mtInformation, [mbOk], 0);}
      Screen.Cursor := crHourglass;
      FileName := TheFileName;
      StringList.Clear;
      StringList.LoadFromFile( FileName );
      SetLineLength;
      PerformSettings;
      Print;
    Finally
      Screen.Cursor := crDefault;
    End;
  End;
End;

{---------------------------------------------------------------------}
Procedure TForm1.FormClose(Sender: TObject; Var Action: TCloseAction);
{---------------------------------------------------------------------}
Begin
  StringList.Free;
End;

End.
